/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH/TECH62
 * Version            : V1.0.0
 * Date               : 2022/08/08
 * Description        : USB FAT12 Disk on 25Q32 Flash Memory over SPI with FatFS.
 * Modified           : 20-September-2024 by Nedelcu Bogdan Sebastian
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *********************************************************************************/

/*********************************************************************
 * @Note
 *     UDisk Example:
 *     This program provides example of UDisk using W25Q32 flash memory over SPI.
 *     The storage is FAT12, also acessed by FatFS to create directory and file.
 *
 */

#include "ch32v30x_usbfs_device.h"
#include <SPI_FLASH.h>
#include <SW_UDISK.h>
#include "debug.h"
#include "ff.h"


FATFS fs; /* FatFs file system object */
FIL fnew; /* file object */
FRESULT res_flash; /* File operation results */
UINT fnum; /* Number of files successfully read and written */
BYTE ReadBuffer[512] = { 0 };/* read buffer */
BYTE WriteBuffer[] = /* write buffer*/
"This is a CH32V307_W25Q32_SPI_FLASH_FATFS_SYSTEM experiment\r\n";

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */

int main(void) {
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
	Delay_Init();
	USART_Printf_Init(115200);
	printf("SystemClk:%d\r\n", SystemCoreClock);

	printf("USBD UDisk Demo - Storage Medium: SPI FLASH \r\n");
	/* SPI flash init */
	FLASH_Port_Init();
	/* FLASH ID check */
	FLASH_IC_Check();

	Delay_Ms(10);

	char filename[20];

	res_flash = f_mount(&fs, "1:", 1);   // Mount the file system to partition 0

	if (res_flash == FR_NO_FILESYSTEM) {                    // No document system
		printf("No file system found, start formatting: ");

		res_flash = f_mkfs("1:", 0, 0);   // Formatting Create a document system

		if (res_flash == FR_OK) {
			printf("successful format\r\n");
			res_flash = f_mount( NULL, "1:", 1); // After full format, make a timetable first
			res_flash = f_mount(&fs, "1:", 1);           // Remount
		} else {
			printf("Format failed\r\n");
			// test failed
		}

	} else if (res_flash == FR_OK) {                      // Mounted successfully
		printf("Test directory creation functionality: ");

		strcpy(filename, "1:/TestDir");

		res_flash = f_mkdir(filename);

		printf("res_flash=%X\r\n", res_flash);

		if (res_flash == FR_OK) {
			printf("Folder created successfully[%s]\r\n", filename);
		} else if (res_flash == FR_EXIST) {
			printf("Folder already exists\r\n");
		} else {
			printf("Folder creation failed: %d\r\n", res_flash);
		}

		printf("Test file writing function: ");

		res_flash = f_open(&fnew, "1:TestDir/TestFile.txt",	FA_CREATE_ALWAYS | FA_WRITE); // Open the file for writing. If the file is not found, create a new file.

		printf("res_flash=%X\r\n", res_flash);

		if (res_flash == FR_OK) {
			res_flash = f_write(&fnew, WriteBuffer, strlen(WriteBuffer), &fnum); // Write writes specified data to the file

			if (res_flash == FR_OK) {
				printf("Successfully written[%d]Bytes of data:[%s]\r\n", fnum, WriteBuffer);
				f_close(&fnew);                                    // close file

				printf("Test file reading function: \r\n");

				res_flash = f_open(&fnew, "1:TestDir/TestFile.txt",	FA_OPEN_EXISTING | FA_READ);       // reopen

				if (res_flash == FR_OK) {
					res_flash = f_read(&fnew, ReadBuffer, sizeof(ReadBuffer), &fnum);

					if (res_flash == FR_OK) {
						printf("Read successfully[%d]Bytes of data:[%s]\r\n", fnum, ReadBuffer);
					} else {
						printf("File read failed: %d\r\n", res_flash);
					}

				} else {
					printf("Failed to open file\r\n");

				}
				f_close(&fnew);

			} else {
				printf("Failed to write data to file: %d\r\n", res_flash);
				f_close(&fnew);                                    // close file
				// test failed
			}

		} else {
			printf("File opening or creation failed\r\n");
			// test failed
		}

	} else {
		printf("File system mount failed: %d\r\n", res_flash);
		printf("Check, please SPI Flash working status\r\n");
		// test failed
	}

	f_mount( NULL, "1:", 1);                              // Unmount file system

	// Enable Udisk
	Udisk_Capability = Flash_Sector_Count;
	Udisk_Status |= DEF_UDISK_EN_FLAG;

	// USBFSD device init
	USBFS_RCC_Init();
	USBFS_Device_Init(ENABLE);

	while (1) {
		;
	}
}

